# *** PACK_GRID PYW - 14 Jul 2020  21:32:20 - JKNAUTH

"""
The Pack Grid application displays the results of a set of tkinter test
cases which are in a user-specified file.  For each test case in the
file, one or more labels are packed in a frame.  Various pack options
can be used for each label packed.

pack_grid.py handles the dialogs to get user input for the test case:
fileid, options, etc.  It calls pack_grid_utilities to analyze the
selected file and to display the test case results.  Some of the larger
dialog setups and event handlers are in pack_grid_handlers.

See pack_grid_documentation.py for details.  You can also click on the
entries at the bottom of the Pack Grid dialog to display this program
information.
"""

version = 2.2

from tkinter import *
from tkinter.messagebox import showerror, showwarning
from tkinter.filedialog import askopenfilename
import sys, os, os.path

import pack_grid_utilities
# There is also a pack_grid_handlers import below the root=Tk().


# ======================================================================
# Set up the geometry of the main window, which is used to display the
# dialogs for user input.  All other displays are Toplevel windows.

root = Tk()
# The import below must follow the above "root = Tk()" to give the
# StringVar() in pack_grid_handlers a window to anchor to.
import pack_grid_handlers

dialog_width  = 375; str_width  = str(dialog_width)
dialog_height = 575; str_height = str(dialog_height)

x_offset = str((root.winfo_screenwidth()  - dialog_width)  // 2)
y_offset = str((root.winfo_screenheight() - dialog_height) // 2)

geom = str_width + 'x' + str_height + '+' + x_offset + '+' + y_offset
root.geometry(geom)

root.title('Pack Grid  (version ' + str(version) + ')')


# ======================================================================
# Miscellaneous display configuration settings

# Settings for the window holding the Library listbox object
lib_x_offset = str(int(x_offset) + dialog_width + 20)
lib_y_offset = y_offset
lib_width = 470                        # Width of Library listbox window

# Settings for the standard header labels for the dialog sections
lbl_width = 47   # Width of header labels (chars)
lbl_wrap = 330   # Text wraplength of header labels (pixels)
lbl_padx =   4   # Padding for text in header labels (pixels)

# Padding above the header labels for each dialog section
opt_row_padyt  =  5               # Over the options checkbox section
fid_row_padyt  = 10               # Over the fileid selection section
info_row_padyt = 10               # Over the information display section



# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
# $$$$$$  Build the  >>> GET OPTIONS <<<  section of the dialogs.  $$$$$
# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

# ----------------------------------------------------------------------
# Create frame for options dialog.  Set options dialog display sizes.

opt_frame = Frame(root)              # Frame for options dialog grid
opt_frame.pack()

opt_button_width  =  9               # Checkbutton width (chars)
opt_button_height  = 1               # Checkbutton height (chars)
opt_ckb_txt_width = 35               # Checkbutton text width (chars)

Label(opt_frame, text="Optionally select one or more of these display options, " +
                 "*THEN* select a test case file in the section below.",
                 bg='cyan', justify=LEFT, anchor=W,
                 width=lbl_width, wraplength=lbl_wrap, padx=lbl_padx
     ).grid(row=0, column=0, columnspan=2, pady=(opt_row_padyt,0))


# ----------------------------------------------------------------------
# Handle the "reduce" option dialog.

# Entered on "reduce" checkbutton check
def gred_onPress():
    global reduce_size                               # Flag for "reduce"
    reduce_size = gred_var.get()
#.......................................................................
# Set up "reduce" option dialog.
gred_var = IntVar()
Checkbutton(opt_frame, text="reduce",
            width=opt_button_width, height=opt_button_height, anchor=W,
            command=gred_onPress, variable=gred_var
     ).grid(row=1, column=0, pady=(opt_row_padyt,0))
Label(opt_frame, text='In grid mode, reduce the display size by about\n' +
                      '1/2. The results window can be moved and/or\n' +
                      'resized by dragging with the mouse.',
      justify=LEFT, anchor=W, width=opt_ckb_txt_width
     ).grid(row=1, column=1, pady=(opt_row_padyt,0))

reduce_size = False            # $$$ Target variable for this dialog $$$

# ----------------------------------------------------------------------
# Handle the "abbreviate" option dialog.

# Entered on "abbreviate" checkbutton check
def gabb_onPress():
    global abbreviate                            # Flag for "abbreviate"
    abbreviate = gabb_var.get()
#.......................................................................
# Set up "abbreviate" option dialog.
gabb_var = IntVar()
Checkbutton(opt_frame, text="abbreviate",
            width=opt_button_width, height=opt_button_height, anchor=W,
            command=gabb_onPress, variable=gabb_var
     ).grid(row=2, column=0, pady=(opt_row_padyt,0))
Label(opt_frame, text='Abbreviate some option and value spellings\n' +
                      'to shorten the labels in the display.',
      justify=LEFT, anchor=W, width=opt_ckb_txt_width
     ).grid(row=2, column=1, pady=(opt_row_padyt,0))

abbreviate = False             # $$$ Target variable for this dialog $$$

# ----------------------------------------------------------------------
# Handle the "just one" option dialog checkbox.

# Entered on "just one" checkbutton check
def gjo_onPress():
    global just_one, case_ID, ID_entry, sel_button
    just_one = gjo_var.get()  # Get current setting of "just one" checkbox
    case_ID = ID_entry.get().upper()   # Get current value in ID entry box
    if just_one:
       sel_button.config(state=NORMAL, bg='yellow')
       ID_entry.config(state=NORMAL, bg='yellow')
       ID_entry.select_range(0,END)
       ID_entry.focus()       # Put focus on ID entry box to prompt for
                              #    a different ID
    else:
       sel_button.config(state=DISABLED, bg='#F0F0F0')
       ID_entry.config(state=DISABLED, bg='#F0F0F0')
#.......................................................................
# Set up "just one" option dialog.
gjo_var = IntVar()
Checkbutton(opt_frame, text="just one",
            width=opt_button_width, height=opt_button_height, anchor=W,
            command=gjo_onPress, variable=gjo_var
     ).grid(row=3, column=0, pady=(opt_row_padyt,0))
Label(opt_frame, text='Process just one test case in a file.  The\n' +
                      'results window can be moved and/or resized\n' +
                      'by dragging with the mouse.',
      justify=LEFT, anchor=W, width=opt_ckb_txt_width
     ).grid(row=3, column=1, pady=(opt_row_padyt,0))

just_one = False               # $$$ Target variable for this dialog $$$

# ----------------------------------------------------------------------
# Handle the "just one" option dialog's ID entry box.

sel_frame = Frame(root)
sel_frame.pack()

# Entered on Enter or Set button press.
def get_ID(event):
    global case_ID, ID_entry, sel_button
    case_ID = ID_entry.get()      # Get ID from entry box
    if case_ID == '':
       case_ID = 'First'
       ID_entry.insert(0,'First')

    check_ID = case_ID.upper()    # Check ID format
    if check_ID != 'FIRST' and (not case_ID.isdigit() or len(case_ID) > 3):
       showerror('ERROR!',
           '"' + case_ID + '" does not have a valid test case ID format.')
       return
    ID_entry.config(state=DISABLED)
    sel_button.config(state=DISABLED, bg='#F0F0F0')
# ......................................................................
# Set up ID entry box and its Set button.
ID_entry = Entry(sel_frame, width=5, justify=CENTER)
ID_entry.grid(row=0, column=0, pady=(opt_row_padyt-3,0), sticky=W)
ID_entry.insert(0, 'First')   # "First" = default ID = use first in file
ID_entry.config(state=DISABLED)
ID_entry.bind('<Return>', get_ID)

sel_button = Button(sel_frame, text='Set', command=(lambda: get_ID(None)),
                    width=2, bg='#F0F0F0', state=DISABLED)
sel_button.grid(row=0, column=1, sticky=W, padx=15)

Label(sel_frame, text='For "just one" mode, check its box, enter a test\n' +
                      'case ID, then press Set or Enter.  Finally, select\n' +
                      'a file in the section below. "First" (the default)\n' +
                      'means use the first ID in the selected file.',
      justify=LEFT, anchor=W, width=opt_ckb_txt_width
     ).grid(row=0, column=2, pady=(opt_row_padyt-3,0))

case_ID = ''                   # $$$ Target variable for this dialog $$$



# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
# $$$$$$  Build the  >>> GET FILEID <<<  section of the dialogs.  $$$$$$
# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

# ----------------------------------------------------------------------
# Create frame for fileid dialog.

fid_frame = Frame(root)
fid_frame.pack()

# ----------------------------------------------------------------------
# Process the fileid dialog.

def gfid_onPress():
    global fileid, ID_entry
    pick = gfid_var.get()

    if ID_entry.cget("state") == NORMAL:
       showwarning('WARNING',
                   'Did you forget to press Set to select the ID?')
       return

    samples_dir = os.getcwd() + '\\Samples'
    test_dir    = os.getcwd() + '\\Unit_Test'

    # Allow user to pick from some standard test case files.
    if pick   == 'Small-size cells (standard test case)':
       file = '$BUILTIN$_small_size_18.txt'
       fileid = os.path.join(samples_dir,file)

    elif pick == 'Medium-size cells (standard test case)':
       file = '$BUILTIN$_medium_size_8.txt'
       fileid = os.path.join(samples_dir,file)

    elif pick == 'Large-size cells (standard test case)':
       file = '$BUILTIN$_large_size_2.txt'
       fileid = os.path.join(samples_dir,file)

    # Allow user to browse for a file.
    elif pick == 'Browse to select a file':
       fileid = askopenfilename(initialdir=samples_dir)
       if fileid == '':
          showerror('Fileid search',
                    'User canceled request to "find some other file"')
          return

    # Help user run the $$$test file du jour$$$ test case file.
    elif pick == 'TESTING:   Select   $$$test file du jour$$$.txt':
       file = '$$$test_du_jour$$$.txt'
       lib_dir = samples_dir
       lib_title = 'Library of Pack Grid Sample Files'

       # $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
       # Go build the library listbox in its Toplevel window.
       pack_grid_handlers.open_library_window(
           lib_dir, lib_title, test_du_jour=file,
           dialog=root, display_func=display_test_cases)
       # $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$


    # Allow user to pick a file from a library of files.
    elif (pick == 'Choose a file from the Samples library' or
          pick == 'TESTING:   Choose a file from the Unit Test library'):

       # Set up for chosen library.
       if pick == 'Choose a file from the Samples library':
          lib_dir = samples_dir
          lib_title = 'Library of Pack Grid Sample Files'
       else:
          lib_dir = test_dir
          lib_title = 'Library of Pack Grid Unit Test Files'

       # $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
       # Go build the library listbox in its Toplevel window.
       pack_grid_handlers.open_library_window(
           lib_dir, lib_title, test_du_jour=None,
           dialog=root, display_func=display_test_cases)
       # $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    # -------------------------------------------------------------------
    # All file selection methods have now been tried or bypassed.  Now
    # go examine the selected test case file (if one was selected) and
    # display the results.
    display_test_cases(fileid)

    fileid = ''  # Ensure the fileid has been reset to "none chosen yet"

    return

#.......................................................................
# Set up fileid dialog to select the test case file.
gfid_var = StringVar()

files = ['Small-size cells (standard test case)',
         'Medium-size cells (standard test case)',
         'Large-size cells (standard test case)',
         'Browse to select a file',
         'Choose a file from the Samples library',
         'TESTING:   Select   $$$test file du jour$$$.txt',
         'TESTING:   Choose a file from the Unit Test library']

Label(fid_frame, text="Click a button to select the test case file " +
                 "and execute the test(s) in it.  " +
                 "(Check any desired above options *FIRST*.)",
                 bg='#FFBBBB', justify=LEFT, anchor=W,
                 width=lbl_width, wraplength=lbl_wrap, padx=lbl_padx
     ).grid(row=0, column=0, pady=(fid_row_padyt,0))

i = 1
for key in files:
    rb = Radiobutton(fid_frame, text=key,
                    command=gfid_onPress,
                    variable=gfid_var,
                    value=key)
    if i == 5:
       rb.config(fg="blue", font=("arial",9,"bold"))
    if i == 6:
       rb.config(fg="green", font=("arial",9,"bold"))
       rb.grid(row=i, pady=(4,0))
    if i == 6 or i == 7:
       rb.config(bg='#D8D8D8', width=45, anchor=W)

    rb.grid(row=i, column=0, sticky=NW)
    i += 1

gfid_var.set('all off')  # Initial setting for radiobuttons -- none set

fileid = ''                    # $$$ Target variable for this dialog $$$



# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
# $$$$$$$  Build the  >>> GET INFO <<<  section of the dialogs.  $$$$$$$
# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

# This dialog lets the user select which info text documentation to
# display.  The event handler is in pack_grid_handlers.  The actual info
# displaying is done from there.

# ----------------------------------------------------------------------
# Create frame for info dialog.  Set info dialog display sizes.

info_frame = Frame(root)         # Frame for info dialog grid
info_frame.pack()

#.......................................................................

Label(info_frame, text="Click a button to display program information.",
                 bg='#FFCC66', justify=LEFT, anchor=W,
                 width=lbl_width, wraplength=lbl_wrap, padx=lbl_padx
     ).grid(row=0, column=0, pady=(info_row_padyt,0))

i = 1
for key in pack_grid_handlers.info:
    Radiobutton(info_frame, text=key,
                      command=pack_grid_handlers.ginfo_onPress,
                      variable=pack_grid_handlers.ginfo_var,
                      value=key
    ).grid(row=i, column=0, sticky=NW)
    i += 1

# Initial setting for radiobuttons -- none set
pack_grid_handlers.ginfo_var.set('all off')



# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
# $$$$  Go analyze the selected file and then display the results.  $$$$
# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

def display_test_cases(fileid):
   global case_ID

   # The case_ID = ALL below is not really used if this call is going to
   # be for a grid mode display setup, but something must always be
   # specified for case_ID to allow the second call below.
   if not just_one: case_ID = 'ALL'

   if fileid != '':

       # Call a utility function to verify this is a good file.  If it
       # is, the function will extract the test case data from the file
       # and save it in the utility module for the next step.
       status = pack_grid_utilities.process_fileid(fileid)

       if status == '':

          display_parms = (case_ID, just_one, reduce_size, abbreviate)

          # Call a utility function to build a display window using the
          # test case data that was just saved.
          pack_grid_utilities.build_display_window(fileid, display_parms)

   # Now return to waiting in the mainloop.
   return


# ======================================================================
# Wait for user action on the dialogs.

mainloop()
