# *** PACK_GRID_DOCUMENTATION PY - 20 Jun 2020  16:29:23 - JKNAUTH

version = 2.2

title_gen_top = """
                      General Information about Pack Grid
"""

propo_gen_top = """
The Pack Grid application displays the output of a set of tkinter test
cases which are in a specified file.  For each test case in the file one
or more labels are packed in a frame.  Various pack options (see below)
can be used for each label packed.  It is easy to add test case files
having test cases with any combination of pack options.  Because you can
display the results of many cases simultaneously in many tailorable
arrangements, it is easy to make comparisons to see what happens when a
pack option is altered.

Below are the valid pack options.  The first option in each group is the
default.  The upper and lower case used must be as is shown here; that's
a pack requirement.  Although tkinter pack accepts strings such as "top"
as well as the symbol form, TOP, Pack Grid requires the symbol form as
shown below.  Other than that, if tkinter pack accepts a string of
options as valid input, that same string can be used in Pack Grid test
cases.  An option need not be present; for those present, their order
and spacing are not significant.
"""

mono_gen_asis = """
           side=TOP,  side=BOTTOM,  side=LEFT,  side=RIGHT

           fill=NONE,  fill=X,  fill=Y,  fill=BOTH

           expand=NO,  expand=YES

           anchor=CENTER,
           anchor=NW,  anchor=N,  anchor=NE,  anchor=E,
           anchor=SE,  anchor=S,  anchor=SW,  anchor=W
"""

propo_gen_bottom = """
A file's test cases are normally all run and the results are displayed
in the order in which the test cases appear in the file.  For each test
case there is an ID and a list of packing options for each label to be
packed for that test case.  A detailed example is shown in the "Main
Data Structures for Pack Grid" document.

Optionally there is a "rows = nbr" line in the file.  It specifies the
desired number of rows in the display.  The display may be one or two
columns wide depending on the number of test cases and the number of
rows in the display.  By default, Pack Grid will try to fill the screen
with the results display, dividing the test cases over two columns as
evenly as possible.  You can override this by specifying the number of
rows to use; then Pack Grid will fill the first column (having nbr rows)
before moving on to the second.

Either all the test cases in the file will be run (grid mode) or just
one will be run ("just one" mode).  The user specifies the ID of the
"just one" test case; the default is the first test case in the selected
file.  In grid mode each test case is allocated a fixed-size cell in a
window grid to display the results of that test.  In "just one" mode the
results for the single test case are displayed in an ungridded window.
For both grid mode and "just one" mode, the results windows can be
resized by dragging with the left mouse button.  This can show how
labels grow, shrink, or are clipped, depending on a window's size.

A dialog option lets the user request a reduced display size for the
grid instead of having it occupy essentially the whole screen (the
default), which can be overwhelming on a large screen.  However the
reduced size means fewer test cases from the file can be displayed.

A dialog option abbreviates some pack options and values in the display
allowing more labels to be visible.  For example expand=YES would be
abbreviated as exp=Y.

Multiple windows can be opened simultaneously, e.g., for multiple test
case files and/or multiple information ("help") windows.
Double-clicking (left mouse button) on a results window or an
information window will close that window as if you had clicked the X in
the upper right corner; all other windows remain open.  However closing
the initial dialog window by clicking its X will close all windows and
will terminate the program.

Via the dialog window you can choose to run one of several "standard"
test case files; these primarily demonstrate displays with various grid
sizes (small, medium, and large).  Of more use, you can browse your PC
to find a test file or navigate a Samples library, which has many
informative test cases.  You can add more files to the library.  There
is also a Unit Test library, which contains test cases used to test Pack
Grid itself, e.g., to make sure it catches certain errors.  Finally,
there is a button to easily select a special "du jour" test case file;
this file can be edited/replaced while you use it to experiment with how
changes in packing options affect the display.

When you are navigating thru the Pack Grid libraries, by using various
mouse clicks you can choose either to run the test cases in a selected
file or you can choose to edit the file.  You can specify your desired
text editor in a PACK_GRID_EDITOR environment variable.  If this
variable is non-existent or is a null string, ***Windows*** file association
is used to select the editor; that means UNIX/LINUX users must specify
an editor in the PACK_GRID_EDITOR variable.  In contrast, Cygwin, for
example, can use Windows file association since the Windows-only
os.startfile method works in the Cygwin (embedded in Windows)
environment.
"""


title_data_top = """
                  Main Data Structures for Pack Grid
"""

propo_data_top = """
When a test case file is processed by the Pack Grid application, a
dictionary is created with one entry per test case in the file.  Each
test case in the file has a unique digit string ID, e.g., '13', which is
essentially used as the key for that test case in the dictionary.  To
facilitate later,processing the dictionary keys are normalized with
leading 0s to a length of three, e.g., '013'.

Except for "text=" lines (see below) each file line of the test case
represents a label to be packed.  The line consists of the test case's
ID followed by pack options for that label, all separated by commas.
This pack options part of the line is the "pack string" for that label.
In the dictionary the value associated with the test case's ID key is a
list of all the pack strings for the labels to be packed in that test
case.  The list keeps the strings in order to represent the order in
which the label packing is to be done.  For example, an ID with two
labels to be packed might have this list of two strings, one string for
each label, which are packed in the order of the strings in the list:
['side=BOTTOM, fill=X', 'side=LEFT, fill=Y, expand=YES'].

Optionally a test can have one or more "text=" lines for commentary
about the test case; no pack options can be on such lines.  The "text="
must be the first non-blank text after the test case's ID.  The data
after the "text=" is put in a special comment label at the bottom of the
display frame for that test case.  If there are multiple such lines for
a test case, all the data is consolidated into one string for the
comment label packed for that test case.  The "text=" lines can be in
any order with respect to the other lines for the test case, but will
result in a single side=BOTTOM comment label.  To ensure that this
special label will occupy the whole bottom row of the test case display,
the program packs the comment label before any other labels in the test
case, no matter where the "text=" lines appeared in the test case file.

A file might look like the example below, having lines for test cases 4,
3, 8, and 3 again.  Note that the ID 3 lines did not have to be all
together; lines with various IDs can be intermingled.  However the data
for all lines with the same ID is gathered and put under that ID key in
the dictionary.  The numerical value of the IDs is not significant for
any ordering.  However a separate ID order list (see below) is used to
have the test cases display in the order in which each ID first occurs
in the file.  When processing the test case file, all leading spaces,
blank lines, # (starting in any column) comment lines, and "* ***"
(starting in column 1) timestamp lines are ignored.

If a pack option (e.g. fill or expand) is not in a file line, its
standard default will be used.  In fact the pack string section of the
line is exactly what is passed to pack; tkinter supplies the defaults.
However Pack Grid does do error checking to ensure that what will be
passed to pack (via an eval) consists entirely of valid pack options,
all of which are case sensitive.
"""

mono_data_asis = """
------------------------------------------------------------------------
# This is the pack.DEMO.txt file in the Pack Grid Samples directory.
#
# Although many pack option default settings are explicitly included
# below, e.g., fill=NONE, that is not required by Pack Grid.  For
# example, the default option anchor=CENTER is not explicitly included,
# but still applies.  Also, the spacing shown here is not required.
# In the results display, one space is put after each comma.
#

   4, text=This is an example of putting a displayable comment in a
   4, text=test case.  The "text=" lines need not be the first lines of
   4, text=the test case to display properly at the bottom of the cell

   4, side=TOP,    expand=NO,  fill=NONE, anchor=W
   4, side=TOP,    expand=NO,  fill=NONE, anchor=E
   4, side=LEFT,   expand=NO,  fill=NONE
   4, side=RIGHT,  expand=YES, fill=BOTH

   4, text=or the "just one" window.  The "text=" lines are collected
   4, text=and wrapped so all the text will fit in its special bottom
   4, text=label.

   3, text=Here is some text for test case ID 3.
   3, side=RIGHT,  expand=YES, fill=Y
   3, side=TOP,    expand=NO,  fill=NONE

   8, side=TOP,    expand=YES, fill=NONE, anchor=SE

   3, side=LEFT,   expand=YES, fill=X
   3, side=BOTTOM, expand=NO,  fill=NONE
------------------------------------------------------------------------

The resulting cases dictionary would look like this.  (For brevity the
full "text=" string for key 004 is not shown.)  Note that the optional
"TEXT=" strings are put first in each test cases packing  list.

    { '004': ['TEXT=This is an ... special bottom label.',
              'side=TOP,    expand=NO,  fill=NONE,  anchor=W',
              'side=TOP,    expand=NO,  fill=NONE,  anchor=E',
              'side=LEFT,   expand=NO,  fill=NONE',
              'side=RIGHT,  expand=YES, fill=BOTH']

      '003': ['TEXT=Here is some text for test case ID 3.',
              'side=RIGHT,  expand=YES, fill=Y',
              'side=TOP,    expand=NO,  fill=NONE',
              'side=LEFT,   expand=YES, fill=X',
              'side=BOTTOM, expand=NO,  fill=NONE']

      '008': ['side=TOP,    expand=YES, fill=NONE,  anchor=SE']
    }
"""

propo_data_bottom = """
There is a separate cases_info dictionary, which is used to gather the
"text=" data, if any, for each test case, i.e., for each ID.  When all
the file lines have been read for a test case, the collected text data
for that ID is consolidated into one big "TEXT=" string.  That string is
then put into the cases dictionary entry for that ID as shown above.
Later that "TEXT=" string is used to create a special side=BOTTOM label
to display the commentary text in that test case's part of the display.

If any line has just an ID or an ID followed by a comma, e.g., "11" or
"11," and the rest of the line is blank, that line is essentially
ignored.  It will create a dictionary test case entry for that ID if one
does not already exist, but it will supply no viewable data when the
test case is displayed.  This can be used to create a blank cell for
spacing in a results display to better horizontally align some cells you
want to easily compare.  See TEXT_EXAMPLES.txt in Samples.

An optional "rows = nbr" file line requests that the number of rows in
the display should be nbr if possible.  The default is "rows = max",
which means use the largest possible number of rows for the display;
this maximum number differs from device to device.  If there are
multiple "rows =" lines in the file, the last one is used.

In grid mode the grid cells are stretched vertically (but not
horizontally) to fill the screen using the number of rows determined by
"rows =".  If "rows = max" (the default), as many rows as possible will
be used.  However there is a minimum cell height constraint, currently
hardcoded as 76 pixels, which restricts the maximum possible number of
rows.

If the specified "rows = nbr" is larger than the number of test cases in
the file, nbr is silently reduced to the actual number of test cases.
If the user requests more rows for the display than are physically
possible, a warning is given and the maximum possible number of rows
is used instead.  If there are more test cases to be displayed than will
fit in two columns for the number of rows used, the excess cases (those
occurring later in the file) are ignored and a warning is given before
the reduced number of test cases is displayed.

In addition to the main dictionary, a test case ID order list is built.
This is a list of IDs (in dictionary key format) in the same order as
the IDs first appear in the test case file.  This list is needed to
preserve that order when the test cases are run since a dictionary does
not preserve the order.

If an ID is repeated in the test case file, only its first occurrence is
recorded in the ID order list.  The test case IDs in the file need not
be in numerical order.  During processing, all lines designated by the
same ID are gathered under that ID (as a key) to produce the dictionary
entry for that key.  In the test case file, the lines for various IDs
can be scrambled/intermingled in any order as shown in the example.

For the above example, the ID order list would be ['004', '003', '008'],
preserving the order in which the IDs first appeared in the file.
"""
